/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <aul.h>
#include <sys/types.h>
#include <unistd.h>

#include "bm_dummy_plugin.h"
#include "bm_dummy_util.h"
#include "bm_log.h"

#ifdef FOR_ON_DEMAND_SERVICES
const bm_callbacks_st *cb;
#endif

/*
 * To get AppID from PID - Needs to be handled by the respective framework owners:
 * The PID might become outdated by the time data is read, which will create problems in mapping the application.
 * Sample API's and fucntion is provided below to get app-id based on available pid.
 * 	char *strAppId = NULL;
 * 	pid_t nProcessId = getpid();
 * 	app_manager_get_app_id(nProcessId, &strAppId);
 */
static char* get_current_appid(int pid, uid_t uid)
{
	ENTER;
	_I("getting caller appid with pid=[%d], uid=[%d]", pid, uid);

	int ret = 0;
	char appid[128] = {0, };
	char *appid_ret = NULL;

	ret = aul_app_get_appid_bypid_for_uid(pid, appid, sizeof(appid), uid);

	if (ret < 0)
		_E("fail to get current appid ret=[%d], appid=%s\n", ret, appid);

	appid_ret = strdup(appid);
	if (appid_ret == NULL)
		_E("Memory Allocation Failed");

	EXIT;
	return appid_ret;
}


static int get_record_count()
{
	ENTER;

/*	implementation for getting record count from db-storage/buffer */

	EXIT;
	return 0;
}

static int get_app_count()
{
	ENTER;

/*	implementation for getting application count from db-storage/buffer */

	EXIT;
	return 0;
}

#ifdef FOR_ON_DEMAND_SERVICES
static int on_service_closing()
{
	ENTER;

	/* create data handle */
	bm_display_st *data_h = calloc(1, sizeof(bm_display_st));
	if (data_h == NULL) {
		_E("memory allocation failed");
		return -1;
	}

	// populate data node here with data & the call the function - provide_feature_data()
	// this function's handler is implemented at battery-monitor server.

	if (cb != NULL && cb->provide_feature_data)
		cb->provide_feature_data(BM_PLUGIN_ERROR_NONE, (void *)data_h);

	// the memory provided to 'data_h' will be set free by server.

	EXIT;
	return 0;
}
#endif /* FOR_ON_DEMAND_SERVICES end */

int init(const bm_callbacks_st *callbacks)
{
	ENTER;

	if (!callbacks) {
		_E("invalid input param");
		return -1;
	}

/*	Initialize the necessary paramters and callbacks if required */

#ifdef FOR_ON_DEMAND_SERVICES
/*	handle callbacks */
	cb = callbacks;
#endif

	EXIT;
	return STATUS_SUCCESS;
}

int deinit()
{
	ENTER;

/*	De-initialize callbacks & parameters */

	EXIT;
	return STATUS_SUCCESS;
}

int get_feature_data(bm_data_h *handle, bm_plugin_data_type_e type)
{
	ENTER;

/*################################################# SAMPLE APPROACH #####################################################################*/
#if 0
>-----Fill in the details of the resource into the handle

	There are two list:
			1) Comprising of the resource usage parameters between a certain start/stop time pair.
			2) Comprising of the application using that resource along with the usage time, nested under the (1) list.

	Populating the structures by making the linked list of the below common structures

	FOR EXAMPLE:
		Structure for "display" feature data.

		typedef struct {
			uint high;		/* Total time in milliseconds during which brightness intensity was high */
			uint low;		/* Total time in milliseconds during which brightness intensity was low */
			uint med;		/* Total time in milliseconds during which brightness intensity was medium */
			time_t start;		/* Start time of feature data collection session */
			time_t stop;		/* Stop time of feature data collection session */
			GSList *atm_list;	/* Application time map('app_time_map_st1') list for all active applications between start & stop */
			GSList *display_list;	/* List of nodes of 'bm_display_st' for next consecutive sessions */
		} bm_display_st;


	Common structures for application time map.

		typedef struct {
			char *app_id;		/* Application Id */
			uint time;		/* Total duration for which application as active */
		} app_time_map_st1;
#endif

	int uid = 0;
	char *app_id = get_current_appid(getpid(), uid);
	if (!app_id) {
		_E("failed to get app_id\n");
		return 0;
	}

	_I("current app_id = %s\n", app_id);

	GSList *display_list = NULL;
	bm_display_st *display_record = NULL;
	app_time_map_st1 *app_record = NULL;

	/* Number of record entries for the resource */
	int rec_count = get_record_count();

	for(int i=0; i<rec_count; i++) {

		display_record = (bm_display_st*)calloc(1,sizeof(bm_display_st));
		if(display_record == NULL) {
			_E("Calloc Failed");
			break;
		}

		/* Fill display record with the values of the bm_display_st */
		display_list = g_slist_append(display_list, display_record);

		GSList *app_list = NULL;
		app_list = display_record->atm_list;

		/* Number of apps in the "atm_list" */
		int app_count = get_app_count();

		for(int j=0; j<app_count; j++) {

			app_record = (app_time_map_st1*)calloc(1,sizeof(app_time_map_st1));
			if(app_record == NULL) {
				_E("Calloc Failed");
				break;
			}

			/* populate application record here with usage values */

			/* Fill the app_record with the values of the app_time_map_st1 */
			app_list = g_slist_append(app_list, app_record);
		}
	}

	*handle = display_list;

	EXIT;
	return STATUS_SUCCESS;
}

static bm_api_st display_api = {
	.init = init,
	.deinit = deinit,
	.get_feature_data = get_feature_data
};

int plugin_init(bm_api_st ** api)
{
	ENTER;

	*api = &display_api;

	EXIT;
	return 0;
}

int plugin_deinit(void)
{
	ENTER;

	EXIT;
	return 0;
}

BM_PLUGIN_ADD(BM_PLUGIN_ID_DISPLAY, NAME, AUTHOR, VERSION, plugin_init, plugin_deinit);
